"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const sf_plugins_core_1 = require("@salesforce/sf-plugins-core");
const Constants_1 = require("../../Constants");
const CodeAnalyzerConfigFactory_1 = require("../../lib/factories/CodeAnalyzerConfigFactory");
const EnginePluginsFactory_1 = require("../../lib/factories/EnginePluginsFactory");
const RuleViewer_1 = require("../../lib/viewers/RuleViewer");
const ActionSummaryViewer_1 = require("../../lib/viewers/ActionSummaryViewer");
const RulesAction_1 = require("../../lib/actions/RulesAction");
const messages_1 = require("../../lib/messages");
const Display_1 = require("../../lib/Display");
const LogEventListener_1 = require("../../lib/listeners/LogEventListener");
const ProgressEventListener_1 = require("../../lib/listeners/ProgressEventListener");
const RulesWriter_1 = require("../../lib/writers/RulesWriter");
class RulesCommand extends sf_plugins_core_1.SfCommand {
    // We don't need the `--json` output for this command.
    static enableJsonFlag = false;
    static summary = (0, messages_1.getMessage)(messages_1.BundleName.RulesCommand, 'command.summary');
    static description = (0, messages_1.getMessage)(messages_1.BundleName.RulesCommand, 'command.description');
    static examples = (0, messages_1.getMessages)(messages_1.BundleName.RulesCommand, 'command.examples');
    // TODO: Remove when we go GA
    static state = (0, messages_1.getMessage)(messages_1.BundleName.Shared, 'label.command-state');
    static flags = {
        workspace: sf_plugins_core_1.Flags.string({
            summary: (0, messages_1.getMessage)(messages_1.BundleName.RulesCommand, 'flags.workspace.summary'),
            description: (0, messages_1.getMessage)(messages_1.BundleName.RulesCommand, 'flags.workspace.description'),
            char: 'w',
            multiple: true,
            delimiter: ',',
        }),
        'rule-selector': sf_plugins_core_1.Flags.string({
            summary: (0, messages_1.getMessage)(messages_1.BundleName.RulesCommand, 'flags.rule-selector.summary'),
            description: (0, messages_1.getMessage)(messages_1.BundleName.RulesCommand, 'flags.rule-selector.description'),
            char: 'r',
            multiple: true,
            delimiter: ',',
            default: ["Recommended"]
        }),
        'config-file': sf_plugins_core_1.Flags.file({
            summary: (0, messages_1.getMessage)(messages_1.BundleName.RulesCommand, 'flags.config-file.summary'),
            description: (0, messages_1.getMessage)(messages_1.BundleName.RulesCommand, 'flags.config-file.description'),
            char: 'c',
            exists: true
        }),
        'output-file': sf_plugins_core_1.Flags.file({
            summary: (0, messages_1.getMessage)(messages_1.BundleName.RulesCommand, 'flags.output-file.summary'),
            description: (0, messages_1.getMessage)(messages_1.BundleName.RulesCommand, 'flags.output-file.description'),
            char: 'f'
        }),
        view: sf_plugins_core_1.Flags.string({
            summary: (0, messages_1.getMessage)(messages_1.BundleName.RulesCommand, 'flags.view.summary'),
            description: (0, messages_1.getMessage)(messages_1.BundleName.RulesCommand, 'flags.view.description'),
            char: 'v',
            options: Object.values(Constants_1.View)
        })
    };
    async run() {
        // TODO: Remove when we go GA
        this.warn((0, messages_1.getMessage)(messages_1.BundleName.Shared, "warning.command-state", [(0, messages_1.getMessage)(messages_1.BundleName.Shared, 'label.command-state')]));
        const parsedFlags = (await this.parse(RulesCommand)).flags;
        const outputFiles = parsedFlags['output-file'] ? [parsedFlags['output-file']] : [];
        const view = parsedFlags.view;
        const dependencies = this.createDependencies(view, outputFiles);
        const action = RulesAction_1.RulesAction.createAction(dependencies);
        const rulesInput = {
            'config-file': parsedFlags['config-file'],
            'output-file': outputFiles,
            'rule-selector': parsedFlags['rule-selector'],
            'workspace': parsedFlags['workspace'],
        };
        await action.execute(rulesInput);
    }
    createDependencies(view, outputFiles) {
        const uxDisplay = new Display_1.UxDisplay(this, this.spinner);
        const dependencies = {
            configFactory: new CodeAnalyzerConfigFactory_1.CodeAnalyzerConfigFactoryImpl(),
            pluginsFactory: new EnginePluginsFactory_1.EnginePluginsFactoryImpl(),
            logEventListeners: [new LogEventListener_1.LogEventDisplayer(uxDisplay)],
            progressListeners: [new ProgressEventListener_1.RuleSelectionProgressSpinner(uxDisplay)],
            actionSummaryViewer: new ActionSummaryViewer_1.RulesActionSummaryViewer(uxDisplay),
            viewer: this.createRulesViewer(view, outputFiles, uxDisplay),
            writer: RulesWriter_1.CompositeRulesWriter.fromFiles(outputFiles)
        };
        return dependencies;
    }
    /**
     * Creates the {@link RuleViewer} that will be called from {@link RulesAction.execute} to display rules.
     * If a view option is set, rules will be displayed in the specified format.
     * If an output file is set, rules will not display.
     * By default, the details display will be used.
     */
    createRulesViewer(view, outputFiles = [], uxDisplay) {
        if (view === Constants_1.View.DETAIL) {
            return new RuleViewer_1.RuleDetailDisplayer(uxDisplay);
        }
        else if (view === Constants_1.View.TABLE) {
            return new RuleViewer_1.RuleTableDisplayer(uxDisplay);
        }
        else if (outputFiles.length > 0) {
            return new RuleViewer_1.RulesNoOpDisplayer();
        }
        return new RuleViewer_1.RuleTableDisplayer(uxDisplay);
    }
}
exports.default = RulesCommand;
//# sourceMappingURL=rules.js.map