"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigAction = void 0;
const code_analyzer_core_1 = require("@salesforce/code-analyzer-core");
const LogWriter_1 = require("../writers/LogWriter");
const WorkspaceUtil_1 = require("../utils/WorkspaceUtil");
const LogEventListener_1 = require("../listeners/LogEventListener");
const ConfigModel_1 = require("../models/ConfigModel");
class ConfigAction {
    dependencies;
    constructor(dependencies) {
        this.dependencies = dependencies;
    }
    async execute(input) {
        // ==== PREPARE USER's CONFIG AND USER's CODE ANALYZER =========================================================
        const userConfig = this.dependencies.configFactory.create(input['config-file']);
        // We always add a Logger Listener to the appropriate listeners list, because we should always be logging.
        const logFileWriter = await LogWriter_1.LogFileWriter.fromConfig(userConfig);
        this.dependencies.actionSummaryViewer.viewPreExecutionSummary(logFileWriter.getLogDestination());
        const logEventLogger = new LogEventListener_1.LogEventLogger(logFileWriter);
        this.dependencies.logEventListeners.push(logEventLogger);
        const userCore = new code_analyzer_core_1.CodeAnalyzer(userConfig);
        // LogEventListeners should start listening as soon as the User Core is instantiated, since it can start emitting
        // relevant events basically immediately.
        this.dependencies.logEventListeners.forEach(listener => listener.listen(userCore));
        const enginePlugins = this.dependencies.pluginsFactory.create();
        const enginePluginModules = userConfig.getCustomEnginePluginModules();
        const userEnginePromises = [
            ...enginePlugins.map(enginePlugin => userCore.addEnginePlugin(enginePlugin)),
            ...enginePluginModules.map(pluginModule => userCore.dynamicallyAddEnginePlugin(pluginModule)),
        ];
        await Promise.all(userEnginePromises);
        // ==== PREPARE DEFAULT CONFIG (with disable_engine settings kept) AND DEFAULT CODE ANALYZER ===================
        // TODO: We are currently only passing in the enginePlugins here which are not all plugins. We need to
        // include the dynamically loaded plugins... but dynamicallyAddEnginePlugin loads and adds and so we never get
        // access to the plugins. We need to update core to separate the concerns so that we just have a
        // dynamicallyLoadEnginePlugin to just return the plugin so that we can add these plugins to this list.
        // And/Or we could just have the Code Analyzer class return a list of the engines that were disabled so that
        // we don't need this helper function here.
        const disabledEngines = getDisabledEngineNames(enginePlugins, new Set(userCore.getEngineNames()));
        const rawConfigOnlyWithEnginesDisabled = { engines: {} };
        for (const engineName of disabledEngines) {
            rawConfigOnlyWithEnginesDisabled.engines[engineName] = { disable_engine: true };
        }
        const defaultConfigWithEnginesDisabled = code_analyzer_core_1.CodeAnalyzerConfig.fromObject(rawConfigOnlyWithEnginesDisabled);
        // The Default config produces two Cores, since we have to run two selections.
        const defaultCoreForAllRules = new code_analyzer_core_1.CodeAnalyzer(defaultConfigWithEnginesDisabled);
        const defaultCoreForSelectRules = new code_analyzer_core_1.CodeAnalyzer(defaultConfigWithEnginesDisabled);
        // Only the File Logger should listen to the Default Cores, since we don't want to bother the user with redundant
        // logs printed to the console.
        logEventLogger.listen(defaultCoreForAllRules);
        logEventLogger.listen(defaultCoreForSelectRules);
        const defaultEnginePromises = [
            ...enginePlugins.map(enginePlugin => defaultCoreForAllRules.addEnginePlugin(enginePlugin)),
            ...enginePlugins.map(enginePlugin => defaultCoreForSelectRules.addEnginePlugin(enginePlugin)),
            // Assumption: Every engine's default configuration is sufficient to allow that engine to be instantiated,
            // or throw a clear error indicating the problem.
            ...enginePluginModules.map(pluginModule => defaultCoreForAllRules.dynamicallyAddEnginePlugin(pluginModule)),
            ...enginePluginModules.map(pluginModule => defaultCoreForSelectRules.dynamicallyAddEnginePlugin(pluginModule)),
        ];
        await Promise.all(defaultEnginePromises);
        // ==== PERFORM RULE SELECTIONS ================================================================================
        const userSelectOptions = input.workspace
            ? { workspace: await (0, WorkspaceUtil_1.createWorkspace)(userCore, input.workspace) }
            : undefined;
        const defaultSelectOptionsForAllRules = input.workspace
            ? { workspace: await (0, WorkspaceUtil_1.createWorkspace)(defaultCoreForAllRules, input.workspace) }
            : undefined;
        const defaultSelectOptionsForSelectRules = input.workspace
            ? { workspace: await (0, WorkspaceUtil_1.createWorkspace)(defaultCoreForSelectRules, input.workspace) }
            : undefined;
        // EngineProgressListeners should start listening right before we call the Cores' `.selectRules()` methods, since
        // that's when progress events can start being emitted.
        this.dependencies.progressEventListeners.forEach(listener => listener.listen(userCore, defaultCoreForAllRules, defaultCoreForSelectRules));
        const [userRules, allDefaultRules, selectedDefaultRules] = await Promise.all([
            // The user Core should respect the user's selection criteria.
            userCore.selectRules(input['rule-selector'], userSelectOptions),
            // One of the Default Cores should always use "all", to minimize the likelihood of a rule's default state
            // being unavailable for comparison.
            defaultCoreForAllRules.selectRules(['all'], defaultSelectOptionsForAllRules),
            // One of the Default Cores should use the same selectors as the User Core, so we know what rules are available
            // by default.
            defaultCoreForSelectRules.selectRules(input['rule-selector'], defaultSelectOptionsForSelectRules)
        ]);
        // After the Cores are done running, the listeners need to be told to stop, since some of them have persistent UI
        // elements or file handlers that must be gracefully ended.
        this.dependencies.progressEventListeners.forEach(listener => listener.stopListening());
        this.dependencies.logEventListeners.forEach(listener => listener.stopListening());
        // ==== CREATE AND WRITE CONFIG YAML ===========================================================================
        // We need the Set of disabled engines plus all engines that returned rules for the user's selection on both the
        // Default and User Cores.
        const relevantEngines = new Set([
            ...disabledEngines,
            ...userRules.getEngineNames(),
            ...selectedDefaultRules.getEngineNames()
        ]);
        const configModel = new ConfigModel_1.AnnotatedConfigModel(userCore, userRules, allDefaultRules, relevantEngines);
        const fileWritten = this.dependencies.writer
            ? await this.dependencies.writer.write(configModel)
            : false;
        if (!fileWritten) {
            this.dependencies.viewer.view(configModel);
        }
        this.dependencies.actionSummaryViewer.viewPostExecutionSummary(logFileWriter.getLogDestination(), fileWritten ? input['output-file'] : undefined);
        return Promise.resolve();
    }
    static createAction(dependencies) {
        return new ConfigAction(dependencies);
    }
}
exports.ConfigAction = ConfigAction;
function getDisabledEngineNames(allEnginePlugins, enabledEngineNames) {
    return allEnginePlugins.flatMap(enginePlugin => enginePlugin.getAvailableEngineNames().filter(engineName => !enabledEngineNames.has(engineName)));
}
//# sourceMappingURL=ConfigAction.js.map